@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: mail.c,v 1.12 2003/08/30 20:20:39 rousseau Exp $ 
+ * $Id: mail.c,v 1.20 2004/07/18 18:06:42 rousseau Exp $ 
  */
 
 #include "config.h"
@@ -33,6 +33,7 @@
 #include <ctype.h>
 #include <gdk/gdkkeysyms.h>
 
+#define GTK_ENABLE_BROKEN
 #include <gtk/gtk.h>
 #include "libplugin.h"
 #include <pi-file.h>
@@ -150,12 +151,12 @@ static void cb_not_yet(GtkWidget * widget, gpointer data)
 }
 #endif
 
-static gboolean prefs_write()
+static gboolean prefs_write(void)
 {
-	gchar fname[255];
+	gchar fname[FILENAME_MAX];
 	FILE *fptr;
 
-	get_home_file_name("jpilot-Mail.rc", fname, 255);
+	get_home_file_name("jpilot-Mail.rc", fname, sizeof(fname));
 	fptr = fopen(fname, "wt");
 
 	if (fptr)
@@ -183,7 +184,7 @@ static gboolean prefs_write()
 	return FALSE;
 }
 
-static gboolean prefs_read()
+static gboolean prefs_read(void)
 {
 	gchar fname[255];
 	FILE *fptr;
@@ -279,7 +280,7 @@ static gboolean prefs_read()
 	return FALSE;
 }
 
-static void prefs_apply()
+static void prefs_apply(void)
 {
 	GtkWidget *entry;
 
@@ -330,16 +331,6 @@ static void prefs_apply()
 		GtkWidget *menu, *item;
 		char *s;
 
-		entry = gtk_object_get_data(GTK_OBJECT(prefs_window), "sync_truncate");
-		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(entry));
-
-		item = gtk_menu_get_active(GTK_MENU(menu));
-		mailSyncPref.truncate =
-			atoi(gtk_object_get_data(GTK_OBJECT(item), "value"));
-
-		jp_logf(JP_LOG_DEBUG,
-			"selected: truncate = %d\n", mailSyncPref.truncate);
-
 		entry = gtk_object_get_data(GTK_OBJECT(prefs_window), "header_encoding");
 		menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(entry));
 
@@ -353,9 +344,9 @@ static void prefs_apply()
 	}
 }
 
-static void prefs_read_pilot()
+static void prefs_read_pilot(void)
 {
-	gchar fname[255];
+	gchar fname[FILENAME_MAX];
 	struct pi_file *pi_fp;
 	int size;
 	int id;
@@ -364,7 +355,7 @@ static void prefs_read_pilot()
 
 	jp_logf(JP_LOG_DEBUG, "Mail: prefs_read_pilot()\n");
 
-	get_home_file_name("Saved Preferences.prc", fname, 255);
+	get_home_file_name("Saved Preferences.prc", fname, sizeof(fname));
 	pi_fp = pi_file_open(fname);
 
 	if (pi_fp)
@@ -467,8 +458,8 @@ static GtkWidget *make_pref_menu(gchar * item_texts[], gchar * sel_text)
 		gchar *text_ptr = item_texts[i];
 
 		menu_item = gtk_radio_menu_item_new_with_label(group, text_ptr);
-		gtk_signal_connect(GTK_OBJECT(menu_item), "activate", cb_pref_menu,
-			text_ptr);
+		gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
+			(GtkSignalFunc)cb_pref_menu, text_ptr);
 		gtk_object_set_data(GTK_OBJECT(menu_item), "value", text_ptr);
 		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menu_item));
 		gtk_menu_append(GTK_MENU(menu), menu_item);
@@ -488,10 +479,6 @@ static GtkWidget *make_pref_menu(gchar * item_texts[], gchar * sel_text)
 	return pref_menu;
 }
 
-gchar *truncate_items[] = {
-	"250", "500", "1000", "2000", "4000", "6000", "8000", NULL
-};
-
 void cb_prefs(GtkWidget * widget, gpointer data)
 {
 	GtkWidget *label, *separator;
@@ -513,6 +500,9 @@ void cb_prefs(GtkWidget * widget, gpointer data)
 		return;
 	}
 
+	/* read prefs set on PDA */
+	prefs_read_pilot();
+
 	prefs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	/*
 	 * gtk_window_set_default_size(GTK_WINDOW(window), 500, 300); 
@@ -746,8 +736,6 @@ void cb_prefs(GtkWidget * widget, gpointer data)
 	 * ******** Sync Options **********
 	 */
 	{
-		GtkWidget *menu;
-		GtkWidget *hbox_tmp;
 		gchar *str;
 
 		label = gtk_label_new("Sync");
@@ -776,18 +764,13 @@ void cb_prefs(GtkWidget * widget, gpointer data)
 		gtk_object_set_data(GTK_OBJECT(prefs_window), "postsync_send",
 			check_postsync_send);
 
-		hbox_tmp = gtk_hbox_new(FALSE, 0);
-		gtk_box_pack_start(GTK_BOX(vbox_sync), hbox_tmp, FALSE, FALSE, 0);
+		label = gtk_label_new("");
+		gtk_box_pack_start(GTK_BOX(vbox_tmp), label, FALSE, FALSE, 0);
 
-		label = gtk_label_new("Truncate to size (bytes):");
-		gtk_box_pack_start(GTK_BOX(hbox_tmp), label, FALSE, FALSE, 0);
-
-		str = g_strdup_printf("%d", mailSyncPref.truncate);
-		menu = make_pref_menu(truncate_items, str);
+		str = g_strdup_printf("Truncate mails to %d bytes (change the truncation size on your PDA)", mailSyncPref.truncate);
+		label = gtk_label_new(str);
 		g_free(str);
-
-		gtk_box_pack_end(GTK_BOX(hbox_tmp), menu, FALSE, FALSE, 0);
-		gtk_object_set_data(GTK_OBJECT(prefs_window), "sync_truncate", menu);
+		gtk_box_pack_start(GTK_BOX(vbox_tmp), label, FALSE, FALSE, 0);
 	}
 
 	/*
@@ -799,9 +782,10 @@ void cb_prefs(GtkWidget * widget, gpointer data)
 	gtk_container_border_width(GTK_CONTAINER(vbox_sig), 5);
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_sig, label);
 
+	label = gtk_label_new("Set the signature on your PDA");
+	gtk_box_pack_start(GTK_BOX(vbox_sig), label, FALSE, FALSE, 0);
+
 	sig_text = gtk_text_new(NULL, NULL);
-	gtk_text_set_editable(GTK_TEXT(sig_text), TRUE);
-	gtk_text_set_word_wrap(GTK_TEXT(sig_text), FALSE);
 
 	if (mailSigPref.signature)
 	{
@@ -911,6 +895,7 @@ void display_records(void)
 	GList *records;
 	GList *temp_list;
 	buf_rec *br;
+	GtkWidget *mail_text;
 
 	jp_logf(JP_LOG_DEBUG, "display_records: show_category = %d\n",
 		show_category);
@@ -922,6 +907,20 @@ void display_records(void)
 	if (glob_mymail_list != NULL)
 		free_mymail_list(&glob_mymail_list);
 
+	if (NULL == mail_main_hbox)
+		/* we are in jpilot-sync, no GUI */
+		return;
+
+	/* clear old message */
+	mail_text = gtk_object_get_data(GTK_OBJECT(mail_main_hbox), "mail_text");
+	gtk_text_freeze(GTK_TEXT(mail_text));
+
+	gtk_text_set_point(GTK_TEXT(mail_text), 0);
+	gtk_text_forward_delete(GTK_TEXT(mail_text),
+		gtk_text_get_length(GTK_TEXT(mail_text)));
+
+	gtk_text_thaw(GTK_TEXT(mail_text));
+
 	mail_clist =
 		gtk_object_get_data(GTK_OBJECT(mail_main_hbox), "mail_clist");
 	gtk_clist_freeze(GTK_CLIST(mail_clist));
@@ -1057,6 +1056,7 @@ void cb_delete(GtkWidget * widget, int data)
 {
 	GtkWidget *mail_clist;
 	struct MyMail *pMail;
+	GtkWidget *mail_text;
 
 	mail_clist = gtk_object_get_data(GTK_OBJECT(mail_main_hbox), "mail_clist");
 	pMail = gtk_clist_get_row_data(GTK_CLIST(mail_clist), glob_row_selected);
@@ -1066,6 +1066,17 @@ void cb_delete(GtkWidget * widget, int data)
 	/* delete the J-Pilot record */
 	mail_delete(pMail, DELETE_FLAG);
 
+	if (NULL != (mail_text = gtk_object_get_data(GTK_OBJECT(mail_main_hbox), "mail_text")))
+	{
+		gtk_text_freeze(GTK_TEXT(mail_text));
+
+		gtk_text_set_point(GTK_TEXT(mail_text), 0);
+		gtk_text_forward_delete(GTK_TEXT(mail_text),
+			gtk_text_get_length(GTK_TEXT(mail_text)));
+
+		gtk_text_thaw(GTK_TEXT(mail_text));
+	}
+
 	/* remove the GTK entry */
 	gtk_clist_remove(GTK_CLIST(mail_clist), glob_row_selected);
 
@@ -1158,7 +1169,7 @@ GtkWidget *make_menu(char *items[], GtkWidget * menu_items[])
 		menu_items[i] = menu_item;
 
 		gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-			cb_menu, GINT_TO_POINTER(i));
+			(GtkSignalFunc)cb_menu, GINT_TO_POINTER(i));
 		group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(menu_item));
 		gtk_menu_append(GTK_MENU(menu), menu_item);
 		gtk_widget_show(menu_item);
@@ -1175,7 +1186,7 @@ GtkWidget *make_menu(char *items[], GtkWidget * menu_items[])
 	return option_menu;
 }
 
-static GtkWidget *make_menus()
+static GtkWidget *make_menus(void)
 {
 	struct MailAppInfo mai;
 	unsigned char *buf;
@@ -1189,16 +1200,20 @@ static GtkWidget *make_menus()
 
 	if (jp_get_app_info("MailDB", &buf, &buf_size) >= 0)
 	{
-
 		unpack_MailAppInfo(&mai, buf, buf_size);
 
 		for (i = 0; i < 16; i++)
 		{
+			char *temp_str;
+			int len;
+
 			if (mai.category.name[i][0] == '\0')
-			{
 				break;
-			}
-			categories[i] = mai.category.name[i];
+
+			temp_str = malloc((len = strlen(mai.category.name[i])*2+1));
+			multibyte_safe_strncpy(temp_str, mai.category.name[i], len);
+			jp_charset_p2j(temp_str, len);
+			categories[i] = temp_str;
 		}
 		categories[i] = NULL;
 
@@ -1210,6 +1225,10 @@ static GtkWidget *make_menus()
 
 	menu_category = make_menu(categories, menu_item_category1);
 
+	if (categories[0])
+		for (i=0; categories[i]; i++)
+			free(categories[i]);
+
 	return menu_category;
 }
 
@@ -1598,7 +1617,7 @@ int plugin_gui(GtkWidget * vbox, GtkWidget * hbox, unsigned int unique_id)
  * record is chosen.  In the latter case, unique ID will be set.  This
  * application should go directly to that record in the case.
  */
-int plugin_gui_cleanup()
+int plugin_gui_cleanup(void)
 {
 	/*
 	 * clean up 
@@ -1643,7 +1662,7 @@ int plugin_startup(jp_startup_info * info)
 	else
 		prefs.folder_path_inbox = g_strdup_printf("%s/nsmail/Inbox", ent->pw_dir);
 
-	prefs.folder_path_sentmail = g_strdup_printf("");
+	prefs.folder_path_sentmail = g_strdup("");
 
 	prefs.default_domain = g_malloc(64);
 	gethostname(prefs.default_domain, 64);
